@@ -8,4 +8,7 @@ from .matplotlib_utils import *
 from .utils import *
 from .imshow_utils import *
 from .encodes import *
-from .minio_lib import *
\ No newline at end of file
+from .minio_lib import *
+from .optimizers import *
+from .tf_utils import *
+from .np_utils import *
