@@ -1,10 +1,27 @@
 import os
+from pathlib import Path
 import shutil
 from os.path import join
-from typing import Tuple, List, Dict
+from typing import Tuple, List, Dict, Union
+from deep_utils.utils.utils.logging_ import log_print
 
 
-def transfer_directory_items(in_dir, out_dir, transfer_list, mode='cp', remove_out_dir=False, skip_transfer=False):
+def transfer_directory_items(in_dir, out_dir, transfer_list, mode='cp', remove_out_dir=False, skip_transfer=False,
+                             remove_in_dir=False):
+    """
+
+    Args:
+        in_dir:
+        out_dir:
+        transfer_list:
+        mode:
+        remove_out_dir:
+        skip_transfer:
+        remove_in_dir: if mode is mv and this is set to true the in_dir will be removed!
+
+    Returns:
+
+    """
     print(f'starting to copying/moving from {in_dir} to {out_dir}')
     if remove_out_dir or os.path.isdir(out_dir):
         remove_create(out_dir)
@@ -19,7 +36,6 @@ def transfer_directory_items(in_dir, out_dir, transfer_list, mode='cp', remove_o
                     print('[INFO] shutil.copy did not find the file, skipping...')
                 else:
                     raise FileNotFoundError()
-
     elif mode == 'mv':
         for name in transfer_list:
             try:
@@ -29,24 +45,41 @@ def transfer_directory_items(in_dir, out_dir, transfer_list, mode='cp', remove_o
                     print('[INFO] shutil.move did not find the file, skipping...')
                 else:
                     raise FileNotFoundError()
+        if remove_in_dir:
+            shutil.rmtree(in_dir)
     else:
         raise ValueError(f'{mode} is not supported, supported modes: mv and cp')
     print(f'finished copying/moving from {in_dir} to {out_dir}')
 
 
 def dir_train_test_split(in_dir, train_dir='./train', val_dir='./val', test_size=0.1, mode='cp', remove_out_dir=False,
-                         skip_transfer=False):
+                         skip_transfer=False, remove_in_dir=False):
     from sklearn.model_selection import train_test_split
     list_ = os.listdir(in_dir)
     train_name, val_name = train_test_split(list_, test_size=test_size)
     transfer_directory_items(in_dir, train_dir, train_name, mode=mode, remove_out_dir=remove_out_dir,
-                             skip_transfer=skip_transfer)
+                             skip_transfer=skip_transfer, remove_in_dir=False)
     transfer_directory_items(in_dir, val_dir, val_name, mode=mode, remove_out_dir=remove_out_dir,
-                             skip_transfer=skip_transfer)
+                             skip_transfer=skip_transfer, remove_in_dir=remove_in_dir)
     return train_name, val_name
 
 
-def split_dir_of_dir(in_dir, train_dir='./train', val_dir='./val', test_size=0.1, mode='cp', remove_out_dir=False):
+def split_dir_of_dir(in_dir, train_dir='./train', val_dir='./val', test_size=0.1, mode='cp', remove_out_dir=False,
+                     remove_in_dir=False):
+    """
+
+    Args:
+        in_dir:
+        train_dir:
+        val_dir:
+        test_size:
+        mode:
+        remove_out_dir:
+        remove_in_dir: if mode is mv and this is set to true the in_dir will be removed!
+
+    Returns:
+
+    """
     if remove_out_dir:
         remove_create(train_dir)
         remove_create(val_dir)
@@ -62,7 +95,9 @@ def split_dir_of_dir(in_dir, train_dir='./train', val_dir='./val', test_size=0.1
             print(f"[INFO] {dir_} is empty, Skipping ...")
             continue
         dir_train_test_split(dir_, train_dir=join(train_dir, data), val_dir=join(val_dir, data), mode=mode,
-                             test_size=test_size, remove_out_dir=remove_out_dir)
+                             test_size=test_size, remove_out_dir=remove_out_dir, remove_in_dir=remove_in_dir)
+    if mode == 'mv' and remove_in_dir:
+        shutil.rmtree(in_dir)
 
 
 def split_xy_dir(x_in_dir,
@@ -74,7 +109,8 @@ def split_xy_dir(x_in_dir,
                  mode='cp',
                  val_size=0.1,
                  skip_transfer=False,
-                 remove_out_dir=False):
+                 remove_out_dir=False,
+                 remove_in_dir=False):
     train_names, val_names = dir_train_test_split(x_in_dir,
                                                   train_dir=x_train_dir,
                                                   val_dir=x_val_dir,
@@ -85,16 +121,25 @@ def split_xy_dir(x_in_dir,
     val_labels = [os.path.splitext(name)[0] + '.txt' for name in val_names]
 
     transfer_directory_items(y_in_dir, y_train_dir,
-                             train_labels, mode=mode, remove_out_dir=remove_out_dir, skip_transfer=skip_transfer)
+                             train_labels, mode=mode, remove_out_dir=remove_out_dir, skip_transfer=skip_transfer,
+                             remove_in_dir=remove_in_dir)
     transfer_directory_items(y_in_dir, y_val_dir, val_labels,
-                             mode=mode, remove_out_dir=remove_out_dir, skip_transfer=skip_transfer)
+                             mode=mode, remove_out_dir=remove_out_dir, skip_transfer=skip_transfer,
+                             remove_in_dir=remove_in_dir)
 
 
-def crawl_directory_dataset(dir_: str, ext_filter: list = None) -> Tuple[List[str], List[int], Dict]:
+def crawl_directory_dataset(dir_: str, ext_filter: list = None, map_labels=False, label_map_dict: dict = None,
+                            logger=None,
+                            verbose=1) -> Union[
+    Tuple[List[str], List[int]], Tuple[List[str], List[int], Dict]]:
     """
     crawls a directory of classes and returns the full path of the items paths and their class names
     :param dir_: path to directory of classes
     :param ext_filter: extensions that will be passed and others will be dropped
+    :param map_labels: This map the labels to indices
+    :param label_map_dict: A map which is used for filtering and also mapping the labels!
+    :param logger: A logger
+    :param verbose: whether print logs or not!
     :return: Tuple[List[str], List[int], Dict], x_list containing the paths, y_list containing the class_names, and
     label_map dictionary.
     """
@@ -107,17 +152,30 @@ def crawl_directory_dataset(dir_: str, ext_filter: list = None) -> Tuple[List[st
             item_path = join(cls_path, item_name)
             name, ext = os.path.splitext(item_name)
             if ext_filter is not None and ext not in ext_filter:
-                print(f"[INFO] {item_path} with {ext} is not in ext_filtering: {ext_filter}")
+                log_print(logger, f"{item_path} with {ext} is not in ext_filtering: {ext_filter}", verbose=verbose)
                 continue
-            x.append(item_path)
-            if cls_name not in label_map:
-                cls_index = len(label_map)
-                label_map[cls_name] = cls_index
+
+            if label_map_dict is not None:
+                if cls_name in label_map_dict:
+                    y.append(label_map_dict[cls_name])
+                else:
+                    log_print(logger, f"Skipping cls_name:{cls_name}, x: {x}")
+                    continue
+            elif map_labels:
+                if cls_name not in label_map:
+                    cls_index = len(label_map)
+                    label_map[cls_name] = cls_index
+                else:
+                    cls_index = label_map[cls_name]
+                y.append(cls_index)
             else:
-                cls_index = label_map[cls_name]
-            y.append(cls_index)
-    print(f"[INFO] successfully crawled {dir_}")
-    return x, y, label_map
+                y.append(cls_name)
+            x.append(item_path)
+    log_print(logger, f"successfully crawled {dir_}", verbose=verbose)
+    if map_labels:
+        return x, y, label_map
+    else:
+        return x, y
 
 
 def remove_create(dir_):
@@ -126,3 +184,35 @@ def remove_create(dir_):
     if os.path.exists(dir_):
         shutil.rmtree(dir_)
     os.makedirs(dir_)
+
+
+def mkdir_incremental(dir_path: str, base_name='exp', fix_name=None) -> Path:
+    """
+    makes new directories, if it exists increment it and makes another one. Good for hyperparameter tuning!
+    Args:
+        dir_path:
+        base_name:
+        fix_name: If provided this will be created!
+
+    Returns:
+
+    """
+    os.makedirs(dir_path, exist_ok=True)
+    if fix_name is not None:
+        final_path = os.path.join(dir_path, fix_name)
+        os.makedirs(final_path, exist_ok=True)
+    else:
+        folders = []
+        for dir_ in os.listdir(dir_path):
+            if base_name in dir_:
+                counter = dir_.split(base_name + '_')[-1]
+                if counter.isdigit():
+                    folders.append(int(counter))
+        if len(folders) == 0:
+            final_path = os.path.join(dir_path, base_name + f"_1")
+        else:
+            max_counter = max(folders)
+            final_path = os.path.join(dir_path, base_name + f"_{max_counter + 1}")
+        os.makedirs(final_path)
+
+    return Path(final_path)
