@@ -1,4 +1,4 @@
-from .download_utils import download_decorator, download_file
+from .download_utils import download_decorator, download_file, unzip_func
 from .main_utils import import_module
 from .lib_decorators import get_from_config, expand_input, get_elapsed_time, rgb2bgr, cast_kwargs_dict
 from .decorators import get_method_time, get_func_time
